@@ -2,7 +2,7 @@
  *
  * common.c - Flickcurl common functions
  *
- * Copyright (C) 2007-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2009, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -61,10 +61,11 @@
 #include <flickcurl.h>
 #include <flickcurl_internal.h>
 
+#include <libxml/xmlsave.h>
 
-const char* const flickcurl_short_copyright_string = "Copyright 2007-2008 David Beckett.";
+const char* const flickcurl_short_copyright_string = "Copyright 2007-2009 David Beckett.";
 
-const char* const flickcurl_copyright_string = "Copyright (C) 2007-2008 David Beckett - http://www.dajobe.org/";
+const char* const flickcurl_copyright_string = "Copyright (C) 2007-2009 David Beckett - http://www.dajobe.org/";
 
 const char* const flickcurl_license_string = "LGPL 2.1 or newer, GPL 2 or newer, Apache 2.0 or newer.\nSee http://librdf.org/flickcurl/ for full terms.";
 
@@ -72,6 +73,10 @@ const char* const flickcurl_home_url_string = "http://librdf.org/flickcurl/";
 const char* const flickcurl_version_string = VERSION;
 
 
+const char* const flickcurl_flickr_service_uri =  "http://www.flickr.com/services/rest/?";
+const char* const flickcurl_flickr_upload_service_uri =  "http://api.flickr.com/services/upload/";
+const char* const flickcurl_flickr_replace_service_uri =  "http://api.flickr.com/services/replace/";
+
 
 static void
 flickcurl_error_varargs(flickcurl* fc, const char *message, 
@@ -116,29 +121,56 @@ flickcurl_write_callback(void *ptr, size_t size, size_t nmemb,
     return 0;
 
   fc->total_bytes += len;
-  
-  if(!fc->xc) {
-    xmlParserCtxtPtr xc;
 
-    xc = xmlCreatePushParserCtxt(NULL, NULL,
-                                 (const char*)ptr, len,
-                                 (const char*)fc->uri);
-    if(!xc)
-      rc=1;
-    else {
-      xc->replaceEntities = 1;
-      xc->loadsubset = 1;
+  if(fc->save_content) {
+    char *b;
+    flickcurl_chunk *chunk;
+    
+    b=(char*)malloc(len);
+    chunk=(flickcurl_chunk*)malloc(sizeof(*chunk));
+    if(b && chunk) {
+      fc->chunks_count++;
+
+      memcpy(b, ptr, len);
+      chunk->content=b;
+      chunk->size=len;
+      chunk->prev= fc->chunks;
+
+      fc->chunks=chunk;
+    } else {
+      if(b)
+        free(b);
+      if(chunk)
+        free(chunk);
+      flickcurl_error(fc, "Out of memory");
     }
-    fc->xc=xc;
-  } else
-    rc=xmlParseChunk(fc->xc, (const char*)ptr, len, 0);
+      
+  }
+  
+  if(fc->xml_parse_content) {
+    if(!fc->xc) {
+      xmlParserCtxtPtr xc;
+
+      xc = xmlCreatePushParserCtxt(NULL, NULL,
+                                   (const char*)ptr, len,
+                                   (const char*)fc->uri);
+      if(!xc)
+        rc=1;
+      else {
+        xc->replaceEntities = 1;
+        xc->loadsubset = 1;
+      }
+      fc->xc=xc;
+    } else
+      rc=xmlParseChunk(fc->xc, (const char*)ptr, len, 0);
 
 #if FLICKCURL_DEBUG > 2
-  fprintf(stderr, "Got >>%s<< (%d bytes)\n", (const char*)ptr, len);
+    fprintf(stderr, "Got >>%s<< (%d bytes)\n", (const char*)ptr, len);
 #endif
 
-  if(rc)
-    flickcurl_error(fc, "XML Parsing failed");
+    if(rc)
+      flickcurl_error(fc, "XML Parsing failed");
+  }
 
 #ifdef CAPTURE
   if(fc->fh)
@@ -164,6 +196,10 @@ flickcurl_new(void)
   if(!fc)
     return NULL;
 
+  fc->service_uri = strdup(flickcurl_flickr_service_uri);
+  fc->upload_service_uri = strdup(flickcurl_flickr_upload_service_uri);
+  fc->replace_service_uri = strdup(flickcurl_flickr_replace_service_uri);
+
   /* DEFAULT delay between requests is 1000ms i.e 1 request/second max */
   fc->request_delay=1000;
   
@@ -269,6 +305,13 @@ flickcurl_free(flickcurl *fc)
   if(fc->upload_value)
     free(fc->upload_value);
 
+  if(fc->service_uri)
+    free(fc->service_uri);
+  if(fc->upload_service_uri)
+    free(fc->upload_service_uri);
+  if(fc->replace_service_uri)
+    free(fc->replace_service_uri);
+
   free(fc);
 }
 
@@ -409,6 +452,80 @@ flickcurl_set_http_accept(flickcurl* fc, const char *value)
 }
 
 
+/**
+ * flickcurl_set_service_uri:
+ * @fc: flickcurl object
+ * @uri: Service URI (or NULL)
+ *
+ * Set Web Service URI for flickcurl requests
+ *
+ * Sets the service to the default (Flickr API web service) if @uri is NULL.
+ */
+void
+flickcurl_set_service_uri(flickcurl *fc, const char *uri)
+{
+  if(!uri)
+    uri = flickcurl_flickr_service_uri;
+    
+#if FLICKCURL_DEBUG > 1
+    fprintf(stderr, "Service URI set to: '%s'\n", uri);
+#endif
+    if(fc->service_uri)
+      free(fc->service_uri);
+    fc->service_uri = strdup(uri);
+}
+
+
+/**
+ * flickcurl_set_upload_service_uri:
+ * @fc: flickcurl object
+ * @uri: Upload Service URI (or NULL)
+ *
+ * Set Web Upload Service URI for flickcurl requests
+ *
+ * Sets the upload service to the default (Flickr API web
+ * upload_service) if @uri is NULL.
+ */
+void
+flickcurl_set_upload_service_uri(flickcurl *fc, const char *uri)
+{
+  if(!uri)
+    uri = flickcurl_flickr_upload_service_uri;
+    
+#if FLICKCURL_DEBUG > 1
+    fprintf(stderr, "Upload Service URI set to: '%s'\n", uri);
+#endif
+    if(fc->upload_service_uri)
+      free(fc->upload_service_uri);
+    fc->upload_service_uri = strdup(uri);
+}
+
+
+/**
+ * flickcurl_set_replace_service_uri:
+ * @fc: flickcurl object
+ * @uri: Replace Service URI (or NULL)
+ *
+ * Set Web Replace Service URI for flickcurl requests
+ *
+ * Sets the replace service to the default (Flickr API web
+ * replace_service) if @uri is NULL.
+ */
+void
+flickcurl_set_replace_service_uri(flickcurl *fc, const char *uri)
+{
+  if(!uri)
+    uri = flickcurl_flickr_replace_service_uri;
+    
+#if FLICKCURL_DEBUG > 1
+    fprintf(stderr, "Replace Service URI set to: '%s'\n", uri);
+#endif
+    if(fc->replace_service_uri)
+      free(fc->replace_service_uri);
+    fc->replace_service_uri = strdup(uri);
+}
+
+
 /**
  * flickcurl_set_api_key:
  * @fc: flickcurl object
@@ -419,7 +536,7 @@ flickcurl_set_http_accept(flickcurl* fc, const char *value)
 void
 flickcurl_set_api_key(flickcurl* fc, const char *api_key)
 {
-#ifdef FLICKCURL_DEBUG
+#if FLICKCURL_DEBUG > 1
   fprintf(stderr, "API Key: '%s'\n", api_key);
 #endif
   if(fc->api_key)
@@ -453,7 +570,7 @@ flickcurl_get_api_key(flickcurl* fc)
 void
 flickcurl_set_shared_secret(flickcurl* fc, const char *secret)
 {
-#ifdef FLICKCURL_DEBUG
+#if FLICKCURL_DEBUG > 1
   fprintf(stderr, "Secret: '%s'\n", secret);
 #endif
   if(fc->secret)
@@ -487,7 +604,7 @@ flickcurl_get_shared_secret(flickcurl* fc)
 void
 flickcurl_set_auth_token(flickcurl *fc, const char* auth_token)
 {
-#ifdef FLICKCURL_DEBUG
+#if FLICKCURL_DEBUG > 1
   fprintf(stderr, "Auth token: '%s'\n", auth_token);
 #endif
   if(fc->auth_token)
@@ -766,7 +883,7 @@ flickcurl_prepare_noauth(flickcurl *fc, const char* method,
   }
   
   return flickcurl_prepare_common(fc,
-                                  "http://www.flickr.com/services/rest/?",
+                                  fc->service_uri,
                                   method,
                                   NULL, NULL,
                                   parameters, count,
@@ -784,7 +901,7 @@ flickcurl_prepare(flickcurl *fc, const char* method,
   }
   
   return flickcurl_prepare_common(fc,
-                                  "http://www.flickr.com/services/rest/?",
+                                  fc->service_uri,
                                   method,
                                   NULL, NULL,
                                   parameters, count,
@@ -907,8 +1024,44 @@ nanosleep(const struct timespec *rqtp, struct timespec *rmtp)
 /* end HAVE_NANOSLEEP */
 
 
-xmlDocPtr
-flickcurl_invoke(flickcurl *fc)
+static size_t 
+flickcurl_curl_header_callback(void* ptr,  size_t  size, size_t nmemb,
+                               void *userdata) 
+{
+  flickcurl* fc=(flickcurl*)userdata;
+  int bytes=size*nmemb;
+
+  /* If flickcurl has already failed, return nothing so that
+   * libcurl will abort the transfer
+   */
+  if(fc->failed)
+    return 0;
+  
+#define EC_HEADER_LEN 17
+#define EM_HEADER_LEN 20
+
+  if(!strncmp((char*)ptr, "X-FlickrErrCode: ", EC_HEADER_LEN)) {
+    fc->error_code=atoi((char*)ptr+EC_HEADER_LEN);
+  } else if(!strncmp((char*)ptr, "X-FlickrErrMessage: ", EM_HEADER_LEN)) {
+    int len=bytes-EM_HEADER_LEN;
+    if(fc->error_msg)
+      free(fc->error_msg);
+    fc->error_msg=(char*)malloc(len+1);
+    strncpy(fc->error_msg, (char*)ptr+EM_HEADER_LEN, len);
+    fc->error_msg[len]='\0';
+    while(fc->error_msg[len-1]=='\r' || fc->error_msg[len-1]=='\n') {
+      fc->error_msg[len-1]='\0';
+      len--;
+    }
+  }
+  
+  return bytes;
+}
+
+
+static int
+flickcurl_invoke_common(flickcurl *fc, char** content_p, size_t* size_p,
+                        xmlDocPtr* docptr_p)
 {
   struct curl_slist *slist=NULL;
   xmlDocPtr doc=NULL;
@@ -916,7 +1069,8 @@ flickcurl_invoke(flickcurl *fc)
 #if defined(OFFLINE) || defined(CAPTURE)
   char filename[200];
 #endif
-
+  int rc=0;
+  
 #if defined(OFFLINE) || defined(CAPTURE)
 
   if(1) {
@@ -936,7 +1090,7 @@ flickcurl_invoke(flickcurl *fc)
     if(access(filename, R_OK)) {
       fprintf(stderr, "Method %s cannot run offline - no %s XML result available\n",
               fc->method, filename);
-      return NULL;
+      return 1;
     }
 #ifdef HAVE_RAPTOR
     uri_string=raptor_uri_filename_to_uri_string(filename);
@@ -952,8 +1106,13 @@ flickcurl_invoke(flickcurl *fc)
 
   if(!fc->uri) {
     flickcurl_error(fc, "No Flickr URI prepared to invoke");
-    return NULL;
+    return 1;
   }
+
+  if(content_p)
+    fc->save_content=1;
+  else
+    fc->xml_parse_content=1;
   
   gettimeofday(&now, NULL);
 #ifndef OFFLINE
@@ -1069,6 +1228,13 @@ flickcurl_invoke(flickcurl *fc)
   if(slist)
     curl_easy_setopt(fc->curl_handle, CURLOPT_HTTPHEADER, slist);
 
+  /* send all headers to this function */
+  curl_easy_setopt(fc->curl_handle, CURLOPT_HEADERFUNCTION, 
+                   flickcurl_curl_header_callback);
+  /* ... using this data pointer */
+  curl_easy_setopt(fc->curl_handle, CURLOPT_WRITEHEADER, fc);
+
+
   if(fc->upload_field) {
     struct curl_httppost* post = NULL;
     struct curl_httppost* last = NULL;
@@ -1108,15 +1274,80 @@ flickcurl_invoke(flickcurl *fc)
 
     /* Requires pointer to a long */
     if(CURLE_OK == 
-       curl_easy_getinfo(fc->curl_handle, CURLINFO_RESPONSE_CODE, &lstatus) )
+       curl_easy_getinfo(fc->curl_handle, CURLINFO_RESPONSE_CODE, &lstatus) ) {
       fc->status_code=lstatus;
+      if(fc->status_code != 200) {
+        if(fc->method)
+          flickcurl_error(fc, "Method %s failed with error %d - %s (HTTP %d)", 
+                          fc->method, fc->error_code, fc->error_msg,
+                          fc->status_code);
+        else
+          flickcurl_error(fc, "Call failed with error %d - %s (HTTP %d)", 
+                          fc->error_code, fc->error_msg,
+                          fc->status_code);
+        fc->failed=1;
+      }
+    }
 
   }
 
   if(slist)
     curl_slist_free_all(slist);
 
-  if(!fc->failed) {
+  if(fc->failed)
+    goto tidy;
+  
+  if(fc->save_content) {
+    char* c;
+    flickcurl_chunk** chunks;
+
+    c=(char*)malloc(fc->total_bytes+1); /* +1 for NUL */
+    chunks=(flickcurl_chunk**)malloc(sizeof(flickcurl_chunk*) * fc->chunks_count);
+    if(c && chunks) {
+      flickcurl_chunk* chunk=fc->chunks;
+      int i;
+      char *p;
+
+      /* create the ordered list of chunks */
+      for(i=fc->chunks_count-1; i >= 0; i--) {
+        chunks[i]=chunk;
+        chunk=chunk->prev;
+      }
+
+      p=c;
+      for(i=0; i < fc->chunks_count; i++) {
+        memcpy(p, chunks[i]->content, chunks[i]->size);
+        p += chunks[i]->size;
+
+        /* free saved chunk once it has been copied */
+        free(chunks[i]->content);
+        free(chunks[i]);
+      }
+      free(chunks);
+
+      /* saved chunks list is now freed */
+      fc->chunks=NULL; 
+      fc->chunks_count=0;
+
+      *p='\0';
+      
+      if(content_p)
+        *content_p=c;
+      else
+        free(c);
+      if(size_p)
+        *size_p=fc->total_bytes;
+      
+    } else {
+      if(c)
+        free(c);
+      if(chunks)
+        free(chunks);
+      flickcurl_error(fc, "Out of memory");
+    }
+  }
+
+  if(fc->xml_parse_content) {
     xmlNodePtr xnp;
     xmlAttr* attr;
     int failed=0;
@@ -1171,12 +1402,16 @@ flickcurl_invoke(flickcurl *fc)
         flickcurl_error(fc, "Call failed with error %d - %s", 
                         fc->error_code, fc->error_msg);
       fc->failed=1;
+    } else {
+      /* pass DOM as an output parameter */
+      if(docptr_p)
+        *docptr_p=doc;
     }
   }
 
   tidy:
   if(fc->failed)
-    doc=NULL;
+    rc=1;
   
 #ifdef CAPTURE
   if(1) {
@@ -1188,7 +1423,27 @@ flickcurl_invoke(flickcurl *fc)
   /* reset special flags */
   fc->sign=0;
   
-  return doc;
+  return rc;
+}
+
+
+xmlDocPtr
+flickcurl_invoke(flickcurl *fc)
+{
+  xmlDocPtr docptr=NULL;
+  if(!flickcurl_invoke_common(fc, NULL, NULL, &docptr))
+    return docptr;
+  return NULL;
+}
+
+
+char*
+flickcurl_invoke_get_content(flickcurl *fc, size_t* size_p)
+{
+  char* content=NULL;
+  if(!flickcurl_invoke_common(fc, &content, size_p, NULL))
+    return content;
+  return NULL;
 }
 
 
@@ -1272,7 +1527,10 @@ flickcurl_xpath_eval(flickcurl *fc, xmlXPathContextPtr xpathCtx,
     goto tidy;
   }
     
-  nodes=xpathObj->nodesetval;
+  nodes = xpathObj->nodesetval;
+  if(xmlXPathNodeSetIsEmpty(nodes))
+    goto tidy;
+  
   for(i=0; i < xmlXPathNodeSetGetLength(nodes); i++) {
     xmlNodePtr node=nodes->nodeTab[i];
     
@@ -1295,6 +1553,64 @@ flickcurl_xpath_eval(flickcurl *fc, xmlXPathContextPtr xpathCtx,
 }
 
 
+char*
+flickcurl_xpath_eval_to_tree_string(flickcurl* fc,
+                                    xmlXPathContextPtr xpathNodeCtx,
+                                    const xmlChar* xpathExpr, size_t* length_p)
+{
+  xmlXPathObjectPtr xpathObj = NULL;
+  xmlNodePtr sd_node;
+  xmlBufferPtr buffer = NULL;
+  xmlSaveCtxtPtr save_ctxt = NULL;
+  char* value = NULL;
+  size_t value_len = 0;
+  xmlNodeSetPtr nodes;
+  
+  xpathObj = xmlXPathEvalExpression(xpathExpr, xpathNodeCtx);
+  if(!xpathObj) {
+    flickcurl_error(fc, "Unable to evaluate XPath expression \"%s\"", 
+                    xpathExpr);
+    fc->failed=1;
+    goto tidy;
+  }
+
+  nodes = xpathObj->nodesetval;
+  if(xmlXPathNodeSetIsEmpty(nodes))
+    goto tidy;
+    
+  sd_node = nodes->nodeTab[0];
+  
+  buffer = xmlBufferCreate();
+  if(!buffer)
+    goto tidy;
+  save_ctxt = xmlSaveToBuffer(buffer, NULL /* encoding */, 0 /* opts */);
+  
+  xmlSaveTree(save_ctxt, sd_node);
+  xmlSaveFlush(save_ctxt);
+  
+  value_len = xmlBufferLength(buffer);
+  if(!value_len)
+    goto tidy;
+  
+  value = (char*)malloc(value_len+1);
+  if(!value)
+    goto tidy;
+  memcpy(value, xmlBufferContent(buffer), value_len+1);
+
+  tidy:
+  if(buffer)
+    xmlBufferFree(buffer);
+
+  if(xpathObj)  
+    xmlXPathFreeObject(xpathObj);
+
+  if(value && length_p)
+    *length_p = value_len;
+
+  return value;
+}
+
+
 /**
  * flickcurl_set_write:
  * @fc: flickcurl object
@@ -1644,3 +1960,222 @@ flickcurl_get_safety_level_from_string(const char* safety_level_string)
 
   return safety_level;
 }
+
+
+#define FEED_FORMAT_COUNT 8
+static struct {
+  const char* name;
+  const char* label;
+  const char* mime_type;
+}
+flickcurl_feed_format_info[FEED_FORMAT_COUNT+1]={
+  { "feed-rss_100", "RSS 1.0", "application/rdf+xml" },
+  { "feed-rss_200", "RSS 2.0", "application/rss+xml" },
+  { "feed-atom_10", "Atom 1.0", "application/atom+xml" },
+  { "feed-georss",  "RSS 2.0 with GeoRSS and W3C Geo for geotagged photos", "application/rss+xml" },
+  { "feed-geoatom", "Atom 1.0 with GeoRSS and W3C Geo for geotagged photos", "application/atom+xml" },
+  { "feed-geordf",  "RSS 1.0 with GeoRSS and W3C Geo for geotagged photos", "application/rdf+xml" },
+  { "feed-kml",     "KML 2.1", "application/vnd.google-earth.kml+xml" },
+  { "feed-kml_nl",  "KML 2.1 network link", "application/vnd.google-earth.kml+xml" },
+  { NULL, NULL, NULL }
+};
+
+
+/**
+ * flickcurl_get_feed_format_info:
+ * @feed_format: input param - feed format index
+ * @name_p: output param - pointer to store feed format name
+ * @label_p: output param - pointer to store feed format label
+ * @mime_type_p: output param - pointer to store feed format mime type
+ * 
+ * Get feed format parameter value information
+ *
+ * As announced 2008-08-25 in
+ * http://code.flickr.com/blog/2008/08/25/api-responses-as-feeds/
+ *
+ * Return value: non-0 if feed_format is out of range
+ **/
+int
+flickcurl_get_feed_format_info(int feed_format,
+                               const char** name_p,
+                               const char** label_p,
+                               const char** mime_type_p)
+{
+  if(feed_format < 0 || feed_format >= FEED_FORMAT_COUNT)
+    return 1;
+
+  if(name_p)
+    *name_p=flickcurl_feed_format_info[feed_format].name;
+
+  if(label_p)
+    *label_p=flickcurl_feed_format_info[feed_format].label;
+
+  if(mime_type_p)
+    *mime_type_p=flickcurl_feed_format_info[feed_format].mime_type;
+
+  return 0;
+}
+
+
+/*
+ * flickcurl_append_photos_list_params:
+ * @list_params: in parameter - photos list paramater
+ * @parameters: in/out parameter - array of name/value parameters
+ * @count_p: in/out parameter - updated as new parameters added
+ * @format_p: out parameter - result format requested or NULL
+ *
+ * INTERNAL - append #flickcurl_photos_list_params to parameter list for API call
+ *
+ * Return value: number of parameters added
+ */
+int
+flickcurl_append_photos_list_params(flickcurl_photos_list_params* list_params,
+                                    const char* parameters[][2], int* count_p,
+                                    const char** format_p)
+{
+  /* NOTE: These are SHARED and pointed to by flickcurl_prepare() to
+   * build the URL */
+  static char per_page_s[4];
+  static char page_s[4];
+  int this_count=0;
+  
+  if(format_p)
+    *format_p=NULL;
+
+  if(!list_params)
+    return 0;
+  
+  if(list_params->extras) {
+    parameters[*count_p][0]  = "extras";
+    parameters[*count_p][1]= list_params->extras;
+    (*count_p)++;
+    this_count++;
+  }
+  if(list_params->per_page) {
+    if(list_params->per_page >= 0 && list_params->per_page <= 999) {
+      sprintf(per_page_s, "%d", list_params->per_page);
+      parameters[*count_p][0]  = "per_page";
+      parameters[*count_p][1]= per_page_s;
+      (*count_p)++;
+      this_count++;
+    }
+  }
+  if(list_params->page) {
+    if(list_params->page >= 0 && list_params->page <= 999) {
+      sprintf(page_s, "%d", list_params->page);
+      parameters[*count_p][0]  = "page";
+      parameters[*count_p][1]= page_s;
+      (*count_p)++;
+      this_count++;
+    }
+  }
+  if(list_params->format) {
+    parameters[*count_p][0]  = "format";
+    parameters[*count_p][1]= list_params->format;
+    (*count_p)++;
+    this_count++;
+
+    if(format_p)
+      *format_p=list_params->format;
+  }
+
+  return this_count;
+}
+
+
+
+#define EXTRAS_FORMAT_COUNT 13
+static struct {
+  const char* name;
+  const char* label;
+}
+flickcurl_extras_format_info[EXTRAS_FORMAT_COUNT+1]={
+  { "date_taken", "Date item was taken"},
+  { "date_upload", "Date item was uploaded"},
+  { "geo", "Geotagging latitude, longitude and accuracy"},
+  { "icon_server", "Item owner icon fields"},
+  { "last_update", "Date item was last updated"},
+  { "license", "Item License "},
+  { "machine_tags", "Machine tags"},
+  { "media", "Item Format: photo or video"},
+  { "o_dims", "Original item dimensions"},
+  { "original_format", "Original item secret and format"},
+  { "owner_name", "Item owner ID"},
+  { "tags", "Item clean tags (safe for HTML, URLs)"},
+  { "views", "Number of times item has been viewed"},
+  { NULL, NULL }
+};
+
+
+/**
+ * flickcurl_get_extras_format_info:
+ * @extras_format: input param - extras format index
+ * @name_p: output param - pointer to store feed format name
+ * @label_p: output param - pointer to store feed format label
+ * 
+ * Get APi extras format parameter value information
+ *
+ * As described 2008-08-19 in
+ * http://code.flickr.com/blog/2008/08/19/standard-photos-response-apis-for-civilized-age/
+ *
+ * Return value: non-0 if extras_format is out of range
+ **/
+int
+flickcurl_get_extras_format_info(int extras_format,
+                                 const char** name_p,
+                                 const char** label_p)
+{
+  if(extras_format < 0 || extras_format >= EXTRAS_FORMAT_COUNT)
+    return 1;
+
+  if(name_p)
+    *name_p=flickcurl_extras_format_info[extras_format].name;
+
+  if(label_p)
+    *label_p=flickcurl_extras_format_info[extras_format].label;
+
+  return 0;
+}
+
+
+/**
+ * flickcurl_photos_list_params_init:
+ * @list_params: photos list params to init
+ *
+ * Initialise an existing photos list parameter structure
+ *
+ * Return value: non-0 on failure
+ */
+int
+flickcurl_photos_list_params_init(flickcurl_photos_list_params* list_params)
+{
+  if(!list_params)
+    return 1;
+  
+  memset(list_params, '\0', sizeof(list_params));
+  list_params->version=1;
+
+  list_params->extras=NULL;
+  list_params->format=NULL;
+  list_params->page= -1;
+  list_params->per_page= -1;
+
+  return 0;
+}
+
+
+/**
+ * flickcurl_search_params_init:
+ * @params: search params to init
+ *
+ * Initialise an existing search parameters structure
+ *
+ * Return value: non-0 on failure
+ */
+int
+flickcurl_search_params_init(flickcurl_search_params* params)
+{
+  memset(params, '\0', sizeof(flickcurl_search_params));
+
+  return 0;
+}
