@@ -4,7 +4,7 @@
  *
  * All API calls and defines here many change in any release.
  *
- * Copyright (C) 2007-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2009, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -63,7 +63,10 @@
   if(!pointer) \
     return; \
 } while(0)
-#define FLICKCURL_ASSERT_OBJECT_POINTER_RETURN_VALUE(pointer, type, ret)
+#define FLICKCURL_ASSERT_OBJECT_POINTER_RETURN_VALUE(pointer, type, ret) do { \
+  if(!pointer) \
+    return ret; \
+} while(0)
 
 #else
 
@@ -109,8 +112,10 @@ int flickcurl_prepare_noauth(flickcurl *fc, const char* method, const char* para
 /* Prepare Flickr API request - POST with form-data parameters */
 int flickcurl_prepare_upload(flickcurl *fc, const char* url, const char* upload_field, const char* upload_value, const char* parameters[][2], int count);
 
-/* Invoke Flickr API at URi prepared above and get back an XML document */
+/* Invoke Flickr API at URi prepared above and get back an XML document DOM */
 xmlDocPtr flickcurl_invoke(flickcurl *fc);
+/* Invoke Flickr API at URi prepared above and get back raw content */
+char* flickcurl_invoke_get_content(flickcurl *fc, size_t* size_p);
 
 /* args.c */
 void flickcurl_free_arg(flickcurl_arg *arg);
@@ -134,9 +139,11 @@ char* flickcurl_sqltimestamp_to_isotime(const char* timestamp);
 
 /* Evaluate an XPath to get the string value */
 char* flickcurl_xpath_eval(flickcurl *fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr);
+char* flickcurl_xpath_eval_to_tree_string(flickcurl* fc, xmlXPathContextPtr xpathNodeCtx, const xmlChar* xpathExpr, size_t* length_p);
 
 char* flickcurl_call_get_one_string_field(flickcurl* fc, const char* key, const char* value, const char* method, const xmlChar* xpathExpr);
 
+int flickcurl_append_photos_list_params(flickcurl_photos_list_params* list_params, const char* parameters[][2], int* count_p, const char** format_p);
 
 /* activity.c */
 flickcurl_activity** flickcurl_build_activities(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* activity_count_p);
@@ -159,15 +166,27 @@ flickcurl_exif** flickcurl_build_exifs(flickcurl* fc, xmlXPathContextPtr xpathCt
 /* group.c */
 flickcurl_group** flickcurl_build_groups(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* group_count_p);
 
+/* institution.c */
+flickcurl_institution** flickcurl_build_institutions(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* institution_count_p);
+flickcurl_institution* flickcurl_build_institution(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr);
+
 /* location.c */
 flickcurl_location* flickcurl_build_location(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr);
 
+/* machinetags.c */
+flickcurl_tag_namespace** flickcurl_build_tag_namespaces(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* namespace_count_p);
+flickcurl_tag_namespace* flickcurl_build_tag_namespace(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* root_xpathExpr);
+flickcurl_tag_predicate_value** flickcurl_build_tag_predicate_values(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int content_mode, int* predicate_value_count_p);
+
 /* user_upload_status.c */
 flickcurl_user_upload_status* flickcurl_build_user_upload_status(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr);
 
 /* md5.c - MD5 as hex string */
 extern char* MD5_string(char *string);
 
+/* members.c */
+flickcurl_member** flickcurl_build_members(flickcurl* fc,  xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* member_count_p);
+
 /* method.c */
 flickcurl_method* flickcurl_build_method(flickcurl* fc, xmlXPathContextPtr xpathCtx);
 
@@ -181,6 +200,7 @@ flickcurl_person* flickcurl_build_person(flickcurl* fc, xmlXPathContextPtr xpath
 /* photo.c */
 flickcurl_photo** flickcurl_build_photos(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* photo_count_p);
 flickcurl_photo* flickcurl_build_photo(flickcurl* fc, xmlXPathContextPtr xpathCtx);
+flickcurl_photos_list* flickcurl_invoke_photos_list(flickcurl* fc, const xmlChar* xpathExpr, const char* format);
 
 /* photoset.c */
 flickcurl_photoset** flickcurl_build_photosets(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* photoset_count_p);
@@ -189,6 +209,11 @@ flickcurl_photoset* flickcurl_build_photoset(flickcurl* fc, xmlXPathContextPtr x
 /* place.c */
 flickcurl_place** flickcurl_build_places(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* place_count_p);
 flickcurl_place* flickcurl_build_place(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr);
+flickcurl_place_type_info** flickcurl_build_place_types(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* place_type_count_p);
+
+/* shape.c */
+flickcurl_shapedata** flickcurl_build_shapes(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* shape_count_p);
+flickcurl_shapedata* flickcurl_build_shape(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr);
 
 /* size.c */
 flickcurl_size** flickcurl_build_sizes(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr, int* size_count_p);
@@ -206,6 +231,15 @@ extern char* my_vsnprintf(const char *message, va_list arguments);
 /* video.c */
 flickcurl_video* flickcurl_build_video(flickcurl* fc, xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr);
 
+struct flickcurl_chunk_s {
+  char* content;
+  size_t size;
+  struct flickcurl_chunk_s *prev;
+};
+
+typedef struct flickcurl_chunk_s flickcurl_chunk;
+
+
 struct flickcurl_s {
   int total_bytes;
 
@@ -290,6 +324,27 @@ struct flickcurl_s {
 #ifdef CAPTURE
   FILE* fh;
 #endif
+
+  /* if non-0 then run content through an XML parser and make a DOM in @xc */
+  int xml_parse_content;
+  
+  /* if non-0 then save content */
+  int save_content;
+
+  /* saved content */
+  /* reverse-ordered list of chunks of data read */
+  flickcurl_chunk* chunks;
+  /* count of chunks */
+  int chunks_count;
+
+  /* Web Service URI that is called */
+  char *service_uri;
+
+  /* Upload Web Service URI that is called */
+  char *upload_service_uri;
+
+  /* Replace Web Service URI that is called */
+  char *replace_service_uri;
 };
 
 struct flickcurl_serializer_s
