@@ -43,18 +43,68 @@
 
 
 /**
- * flickcurl_interestingness_getList:
+ * flickcurl_interestingness_getList_params:
  * @fc: flickcurl context
  * @date: A specific date, formatted as YYYY-MM-DD, to return interesting photos for. (or NULL)
- * @extras: A comma-delimited list of extra information to fetch for each returned record. Currently supported fields are: <code>license</code>, <code>date_upload</code>, <code>date_taken</code>, <code>owner_name</code>, <code>icon_server</code>, <code>original_format</code>, <code>last_update</code>, <code>geo</code>, <code>tags</code>, <code>machine_tags</code>. (or NULL)
- * @per_page: Number of photos to return per page default 100, max 500
- * @page: The page of results to return, default 1
+ * @list_params: #flickcurl_photos_list_params result parameters (or NULL)
  * 
  * Returns the list of interesting photos for the most recent day or a user-specified date.
  *
  * Optional extra type 'media' that will return an extra media=VALUE
  * for VALUE "photo" or "video".  API addition 2008-04-07.
  *
+ * Return value: non-0 on failure
+ **/
+flickcurl_photos_list*
+flickcurl_interestingness_getList_params(flickcurl* fc, const char* date,
+                                         flickcurl_photos_list_params* list_params)
+{
+  const char* parameters[12][2];
+  int count=0;
+  flickcurl_photos_list* photos_list=NULL;
+  const char* format=NULL;
+
+  /* API parameters */
+  if(date) {
+    parameters[count][0]  = "date";
+    parameters[count++][1]= date;
+  }
+
+  /* Photos List parameters */
+  flickcurl_append_photos_list_params(list_params, parameters, &count, &format);
+
+  parameters[count][0]  = NULL;
+
+  if(flickcurl_prepare(fc, "flickr.interestingness.getList", parameters, count))
+    goto tidy;
+
+  photos_list=flickcurl_invoke_photos_list(fc,
+                                           (const xmlChar*)"/rsp/photos/photo",
+                                           format);
+
+  tidy:
+  if(fc->failed) {
+    if(photos_list)
+      flickcurl_free_photos_list(photos_list);
+    photos_list=NULL;
+  }
+
+  return photos_list;
+}
+
+
+/**
+ * flickcurl_interestingness_getList:
+ * @fc: flickcurl context
+ * @date: A specific date, formatted as YYYY-MM-DD, to return interesting photos for. (or NULL)
+ * @extras: A comma-delimited list of extra information to fetch for each returned record. See #flickcurl_photos_list_params for the full list (or NULL)
+ * @per_page: Number of photos to return per page default 100, max 500
+ * @page: The page of results to return, default 1
+ * 
+ * Returns the list of interesting photos for the most recent day or a user-specified date.
+ *
+ * See flickcurl_interestingness_getList() for full description of arguments.
+ *
  * Implements flickr.interestingness.getList (0.13)
  * 
  * Return value: non-0 on failure
@@ -62,59 +112,24 @@
 flickcurl_photo**
 flickcurl_interestingness_getList(flickcurl* fc, const char* date, const char* extras, int per_page, int page)
 {
-  const char* parameters[11][2];
-  int count=0;
-  xmlDocPtr doc=NULL;
-  xmlXPathContextPtr xpathCtx=NULL; 
-  char page_str[10];
-  char per_page_str[10];
-  flickcurl_photo** photos=NULL;
+  flickcurl_photos_list_params list_params;
+  flickcurl_photos_list* photos_list;
+  flickcurl_photo** photos;
+  
+  memset(&list_params, '\0', sizeof(list_params));
+  list_params.format   = NULL;
+  list_params.extras   = extras;
+  list_params.per_page = per_page;
+  list_params.page     = page;
 
-  if(date) {
-    parameters[count][0]  = "date";
-    parameters[count++][1]= date;
-  }
-  if(extras) {
-    parameters[count][0]  = "extras";
-    parameters[count++][1]= extras;
-  }
-  if(per_page >= 0) {
-    sprintf(per_page_str, "%d", per_page);
-    parameters[count][0]  = "per_page";
-    parameters[count++][1]= per_page_str;
-  }
-  if(page >= 0) {
-    sprintf(page_str, "%d", page);
-    parameters[count][0]  = "page";
-    parameters[count++][1]= page_str;
-  }
+  photos_list=flickcurl_interestingness_getList_params(fc, date, &list_params);
+  if(!photos_list)
+    return NULL;
 
-  parameters[count][0]  = NULL;
+  photos=photos_list->photos; photos_list->photos=NULL;  
+  /* photos array is now owned by this function */
 
-  if(flickcurl_prepare(fc, "flickr.interestingness.getList", parameters, count))
-    goto tidy;
-
-  doc=flickcurl_invoke(fc);
-  if(!doc)
-    goto tidy;
-
-
-  xpathCtx = xmlXPathNewContext(doc);
-  if(!xpathCtx) {
-    flickcurl_error(fc, "Failed to create XPath context for document");
-    fc->failed=1;
-    goto tidy;
-  }
-
-  photos=flickcurl_build_photos(fc, xpathCtx,
-                                (const xmlChar*)"/rsp/photos/photo", NULL);
-
-  tidy:
-  if(xpathCtx)
-    xmlXPathFreeContext(xpathCtx);
-
-  if(fc->failed)
-    photos=NULL;
+  flickcurl_free_photos_list(photos_list);
 
   return photos;
 }
