@@ -67,19 +67,19 @@ static const char* flickcurl_photo_field_label[PHOTO_FIELD_LAST+1]={
   "visibility_ispublic",
   "secret",
   "originalsecret",
-  "location_neighborhood",
+  "location_neighbourhood",
   "location_locality",
   "location_county",
   "location_region",
   "location_country",
   "location_placeid",
-  "neighborhood_placeid",
+  "neighbourhood_placeid",
   "locality_placeid",
   "county_placeid",
   "region_placeid",
   "country_placeid",
   "location_woeid",
-  "neighborhood_woeid",
+  "neighbourhood_woeid",
   "locality_woeid",
   "county_woeid",
   "region_woeid",
@@ -349,7 +349,13 @@ static struct {
   ,
   {
     (const xmlChar*)"./location/neighborhood",
-    PHOTO_FIELD_location_neighborhood,
+    PHOTO_FIELD_location_neighbourhood,
+    VALUE_TYPE_STRING
+  }
+  ,
+  {
+    (const xmlChar*)"./location/neighbourhood",
+    PHOTO_FIELD_location_neighbourhood,
     VALUE_TYPE_STRING
   }
   ,
@@ -361,7 +367,13 @@ static struct {
   ,
   {
     (const xmlChar*)"./location/neighborhood",
-    PHOTO_FIELD_location_neighborhood,
+    PHOTO_FIELD_location_neighbourhood,
+    VALUE_TYPE_STRING
+  }
+  ,
+  {
+    (const xmlChar*)"./location/neighbourhood",
+    PHOTO_FIELD_location_neighbourhood,
     VALUE_TYPE_STRING
   }
   ,
@@ -477,13 +489,25 @@ static struct {
   ,
   {
     (const xmlChar*)"./location/neighborhood/@place_id",
-    PHOTO_FIELD_neighborhood_placeid,
+    PHOTO_FIELD_neighbourhood_placeid,
+    VALUE_TYPE_STRING
+  }
+  ,
+  {
+    (const xmlChar*)"./location/neighbourhood/@place_id",
+    PHOTO_FIELD_neighbourhood_placeid,
     VALUE_TYPE_STRING
   }
   ,
   {
     (const xmlChar*)"./location/neighborhood/@woeid",
-    PHOTO_FIELD_neighborhood_woeid,
+    PHOTO_FIELD_neighbourhood_woeid,
+    VALUE_TYPE_STRING
+  }
+  ,
+  {
+    (const xmlChar*)"./location/neighbourhood/@woeid",
+    PHOTO_FIELD_neighbourhood_woeid,
     VALUE_TYPE_STRING
   }
   ,
@@ -523,18 +547,6 @@ static struct {
     VALUE_TYPE_STRING
   }
   ,
-  {
-    (const xmlChar*)"./location/region/@place_id",
-    PHOTO_FIELD_region_placeid,
-    VALUE_TYPE_STRING
-  }
-  ,
-  {
-    (const xmlChar*)"./location/region/@woeid",
-    PHOTO_FIELD_region_woeid,
-    VALUE_TYPE_STRING
-  }
-  ,
   {
     (const xmlChar*)"./location/country/@place_id",
     PHOTO_FIELD_country_placeid,
@@ -640,6 +652,10 @@ flickcurl_build_photos(flickcurl* fc, xmlXPathContextPtr xpathCtx,
       if(!string_value)
         continue;
 
+#if FLICKCURL_DEBUG > 1
+        fprintf(stderr, "  type %d  string value '%s'\n", datatype,
+                string_value);
+#endif
       switch(datatype) {
         case VALUE_TYPE_PHOTO_ID:
           photo->id=string_value;
@@ -669,10 +685,10 @@ flickcurl_build_photos(flickcurl* fc, xmlXPathContextPtr xpathCtx,
 
           if(unix_time >= 0) {
             char* new_value=flickcurl_unixtime_to_isotime(unix_time);
-  #if FLICKCURL_DEBUG > 1
+#if FLICKCURL_DEBUG > 1
             fprintf(stderr, "  date from: '%s' unix time %ld to '%s'\n",
-                    value, (long)unix_time, new_value);
-  #endif
+                    string_value, (long)unix_time, new_value);
+#endif
             free(string_value);
             string_value= new_value;
             int_value= (int)unix_time;
@@ -779,3 +795,98 @@ flickcurl_free_photos(flickcurl_photo** photos)
     flickcurl_free_photo(photos[i]);
   free(photos);
 }
+
+
+flickcurl_photos_list*
+flickcurl_invoke_photos_list(flickcurl* fc, const xmlChar* xpathExpr,
+                             const char* format)
+{
+  flickcurl_photos_list* photos_list=NULL;
+  xmlXPathContextPtr xpathCtx=NULL;
+  const char *nformat;
+  size_t format_len;
+
+  photos_list=(flickcurl_photos_list*)calloc(1, sizeof(flickcurl_photos_list));
+  if(!photos_list) {
+    fc->failed=1;
+    goto tidy;
+  }
+
+  if(format) {
+    nformat=format;
+    format_len=strlen(format);
+  
+    photos_list->content=flickcurl_invoke_get_content(fc,
+                                                      &photos_list->content_length);
+    if(!photos_list->content) {
+      fc->failed=1;
+      goto tidy;
+    }
+
+  } else {
+    xmlDocPtr doc=NULL;
+
+    nformat="xml";
+    format_len=3;
+    
+    doc=flickcurl_invoke(fc);
+    if(!doc)
+      goto tidy;
+
+    xpathCtx = xmlXPathNewContext(doc);
+    if(!xpathCtx) {
+      flickcurl_error(fc, "Failed to create XPath context for document");
+      fc->failed=1;
+      goto tidy;
+    }
+
+    photos_list->photos=flickcurl_build_photos(fc, xpathCtx, xpathExpr,
+                                               &photos_list->photos_count);
+    if(!photos_list->photos) {
+      fc->failed=1;
+      goto tidy;
+    }
+
+  }
+
+
+  photos_list->format=(char*)malloc(format_len+1);
+  if(!photos_list->format) {
+    fc->failed=1;
+    goto tidy;
+  }
+  memcpy(photos_list->format, nformat, format_len+1);
+
+  tidy:
+  if(xpathCtx)
+    xmlXPathFreeContext(xpathCtx);
+
+  if(fc->failed) {
+    if(photos_list)
+      flickcurl_free_photos_list(photos_list);
+    photos_list=NULL;
+  }
+
+  return photos_list;
+}
+
+
+/**
+ * flickcurl_free_photos_list:
+ * @photos_list: photos list object
+ *
+ * Destructor for photos list
+ */
+void
+flickcurl_free_photos_list(flickcurl_photos_list* photos_list)
+{
+  FLICKCURL_ASSERT_OBJECT_POINTER_RETURN(photos_list, flickcurl_photos_list);
+
+  if(photos_list->format)
+    free(photos_list->format);
+  if(photos_list->photos)
+    flickcurl_free_photos(photos_list->photos);
+  if(photos_list->content)
+    free(photos_list->content);
+  free(photos_list);
+}
