@@ -46,7 +46,7 @@
 #define DCTERMS_NS  "http://purl.org/dc/terms/"
 /* predicates dc:creator dc:dateSubmitted dc:rights dc:modified dc:issued
  * dc:created dc:description dc:title */
-#define FOAF_NS     "http://xmlns.com/foaf/0.1/#"
+#define FOAF_NS     "http://xmlns.com/foaf/0.1/"
 /* predicates foaf:maker foaf:name foaf:nick
  * classes foaf:Person foaf:Image */
 #define GEO_NS      "http://www.w3.org/2003/01/geo/wgs84_pos#"
@@ -661,7 +661,7 @@ flickcurl_serialize_photo(flickcurl_serializer* fcs, flickcurl_photo* photo)
       fsf->emit_triple(fcs->data,
                        place_bnode, FLICKCURL_TERM_TYPE_BLANK,
                        PLACES_NS, "type",
-                       flickcurl_get_place_type_label(i),
+                       flickcurl_get_place_type_label((flickcurl_place_type)i),
                        FLICKCURL_TERM_TYPE_LITERAL,
                        NULL);
 
