@@ -43,6 +43,59 @@
 
 
 
+/**
+ * flickcurl_tags_getClusterPhotos:
+ * @fc: flickcurl context
+ * @tag: The tag that this cluster belongs to.
+ * @cluster_id: The top three tags for the cluster, separated by dashes (just like the url).
+ * @list_params: #flickcurl_photos_list_params result parameters (or NULL)
+ * 
+ * Returns the first 24 photos for a given tag cluster
+ *
+ * Implements flickr.tags.getClusterPhotos (1.7)
+ *
+ * Added to API (not announced) some time between 2008-09-04 and 2008-09-19
+ * 
+ * Return value: non-0 on failure
+ **/
+flickcurl_photos_list*
+flickcurl_tags_getClusterPhotos(flickcurl* fc, const char* tag,
+                                const char* cluster_id,
+                                flickcurl_photos_list_params* list_params)
+{
+  const char* parameters[10][2];
+  int count=0;
+  flickcurl_photos_list* photos_list=NULL;
+  const char* format=NULL;
+  
+  if(!tag || !cluster_id)
+    return NULL;
+
+  parameters[count][0]  = "tag";
+  parameters[count++][1]= tag;
+  parameters[count][0]  = "cluster_id";
+  parameters[count++][1]= cluster_id;
+
+  parameters[count][0]  = NULL;
+
+  if(flickcurl_prepare(fc, "flickr.tags.getClusterPhotos", parameters, count))
+    goto tidy;
+
+  photos_list=flickcurl_invoke_photos_list(fc,
+                                           (const xmlChar*)"/rsp/photos/photo",
+                                           format);
+  tidy:
+  if(fc->failed) {
+    if(photos_list)
+      flickcurl_free_photos_list(photos_list);
+    photos_list=NULL;
+  }
+
+  return photos_list;
+}
+
+
+
 /**
  * flickcurl_tags_getClusters:
  * @fc: flickcurl context
@@ -68,7 +121,7 @@ flickcurl_tags_getClusters(flickcurl* fc, const char* tag)
   int count=0;
   xmlDocPtr doc=NULL;
   xmlXPathContextPtr xpathCtx=NULL; 
-  void* result=NULL;
+  flickcurl_tag_clusters* clusters=NULL;
   
   if(!tag)
     return NULL;
@@ -93,17 +146,17 @@ flickcurl_tags_getClusters(flickcurl* fc, const char* tag)
     goto tidy;
   }
 
-  result=flickcurl_build_tag_clusters(fc, xpathCtx,
-                                      (const xmlChar*)"/rsp/clusters/cluster");
+  clusters = flickcurl_build_tag_clusters(fc, xpathCtx,
+                                          (const xmlChar*)"/rsp/clusters/cluster");
 
   tidy:
   if(xpathCtx)
     xmlXPathFreeContext(xpathCtx);
 
   if(fc->failed)
-    result=NULL;
+    clusters = NULL;
 
-  return result;
+  return clusters;
 }
 
 
